#!/bin/bash
# script assume that instal folder is called inst and exists in a build folder

if hash cmake 2>/dev/null; then

# defaults
INST_DIR_NAME=inst
BUILD_DIR_NAME=build
MAKE_THREADS=2
SIGN_PACKAGE=NO
DO_REBUILD="False"
BRANCH_NAME="development"

while getopts "d:i:b:t:rs" option ; do
	case "${option}"
	in
	b) BRANCH_NAME=${OPTARG};;
	i) INST_DIR_NAME=${OPTARG};;
	d) BUILD_DIR_NAME=${OPTARG};;
	t) MAKE_THREADS=${OPTARG};;
	r) DO_REBUILD="True";;
	s) SIGN_PACKAGE=YES;;
	\?)
		echo "err: Invalid option: -$OPTARG" >$2
		exit 1
		;;
	:)
		echo "err: Option -$OPTARG requires an argument." >$2
		exit 1
		;;
	esac
done

# step 1:
cd slicer-build-environment
readonly MY_ENV_DIR=$(cd "$(dirname "$0")"; pwd)
echo 'Step 1 working directory is: ' $MY_ENV_DIR
BUILD_DIR=${MY_ENV_DIR}/${BUILD_DIR_NAME}
if [ -d $BUILD_DIR ] && [ "${DO_REBUILD}" = "True" ]
then
rm -rf $BUILD_DIR
echo 'Removing old build - rebuild flag is on'
fi
mkdir -p ${BUILD_DIR}
cd ${BUILD_DIR}
${MY_ENV_DIR}/env_osx.sh
cmake .. -DCMAKE_INSTALL_PREFIX=${BUILD_DIR}/${INST_DIR_NAME} -DCMAKE_BUILD_TYPE=Release
make -j ${MAKE_THREADS}
echo 'Step 1 Finished!'
# step 2:
cd ${BUILD_DIR}/../../slicer-build/
readonly MY_DIR=$(cd "$(dirname "$0")"; pwd)
echo 'Step 2 working directory is: ' $MY_DIR
BUILD_DIR=${MY_DIR}/${BUILD_DIR_NAME}
if [ -d $BUILD_DIR ] && [ "${DO_REBUILD}" = "True" ]
then
rm -rf $BUILD_DIR	
fi
mkdir -p ${BUILD_DIR}
cd ${BUILD_DIR}
mkdir -p ${BUILD_DIR}/${INST_DIR_NAME}
# copy step1 inst
cp -avf ${MY_ENV_DIR}/${BUILD_DIR_NAME}/${INST_DIR_NAME}/. ${BUILD_DIR}/${INST_DIR_NAME}/
${MY_DIR}/env_osx.sh
cmake .. -DCMAKE_PREFIX_PATH=${BUILD_DIR}/${INST_DIR_NAME} -DCMAKE_BUILD_TYPE=Release -DCURA_VERSION_MAJOR="1" -DCURA_VERSION_MINOR="2" -DCURA_VERSION_PATCH="0" -DBRANCH_NAME="${BRANCH_NAME}" -DBUILD_64BIT=ON -DSIGN_PACKAGE=$SIGN_PACKAGE ..
make -j ${MAKE_THREADS}
else
	echo "cmake is not installed! - Aborted"
	exit 1
fi
exit 0