# Copyright (c) 2017 vabi vabi vabi
# this->Class is released under the terms of the AGPLv3 or higher.

# To fix: set 1chunk = 1px, write better rendering (with closer camera)

import math

from cura.SIProxies.AutoarrangePlugin.Autoarranger import Autoarranger

from PyQt5.QtCore import pyqtSlot
from PyQt5.QtCore import QObject

from UM.Qt.Bindings.MainWindow import MainWindow
from UM.Application import Application
from UM.Scene.Camera import Camera
from UM.Math.Vector import Vector
from UM.Math.Matrix import Matrix
from UM.Operations.TranslateOperation import TranslateOperation
from UM.Operations.GroupedOperation import GroupedOperation

from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
from UM.View.AutoarrangePass import AutoarrangePass

# If model bitmap is to small (too thin to render) resizes bitmap to correct size.
def normalize_bitmaps(models_list, px_to_mm):
    result = list()
    for model in models_list:
        model_size = model[2]
        normalized_px_size = tuple(math.ceil(x*px_to_mm) for x in model_size)
        model_image = model[1].resize(normalized_px_size)
        result.append((model[0], model_image, model[2]))
    return result

def get_models_data():
    previous_positions = list()
    scene_root = Application.getInstance().getController().getScene().getRoot()
    for node in DepthFirstIterator(scene_root):
        if node.isSelectable() and node.getMeshData():
            world_position = node.getWorldPosition()
            bounding_box = node.getBoundingBox()
            y = world_position.y - bounding_box.bottom
            previous_positions.append((id(node), node.getPosition()))
            node.setPosition(Vector(0, y, 0))
    
    render = Application.getInstance().getRenderer()
    result = AutoarrangePass(render.getViewportWidth(), render.getViewportHeight()).render()

    # Move nodes to previous position.
    for node in DepthFirstIterator(scene_root):
        if node.isSelectable() and node.getMeshData():
            node_id = id(node)
            for position in previous_positions:
                if position[0] == node_id:
                    node.setPosition(position[1])

    return result

class AutoarrangeTool(QObject):
    instance = None

    def __init__(self):
        super().__init__()
        root = Application.getInstance().getController().getScene().getRoot()
        self.__camera = Camera("autoarranger_cam", root)
        self.__camera.setPerspective(False)
        self.__camera.setPosition(Vector(0, 3, 0))
        self.__camera.lookAt(Vector(0, 0, 0))

        # Set camera projection matrix by updating it in MainWindow class.
        main_window = Application.getInstance().getMainWindow()
        size = Application.getInstance().getRenderer().getWindowSize()
        main_window._updateViewportGeometry(size[0], size[1])
        # Application.getInstance().getController().getScene().setActiveCamera("autoarranger_cam")

    @pyqtSlot()
    def arrange(self):
        print("Arranging")
        models_list = get_models_data()
        if models_list:
            orient_face = Application.getInstance().getController().getTool("GenceOrientFaceToPlaneTool")
            if orient_face:
                orient_face.invalidateMeshCache()
            build_volume = Application.getInstance().getBuildVolume()
            global_container = Application.getInstance().getGlobalContainerStack()
            table_width = global_container.getProperty("machine_width", "value")
            table_depth = global_container.getProperty("machine_depth", "value")
            # table_margin_size = math.ceil(build_volume._getEdgeDisallowedSize())
            # Set to 10 because of client's delivered specification
            table_margin_size = 10
            table_width = table_width - table_margin_size*2
            table_depth = table_depth - table_margin_size*2
            table_size = (table_width, table_depth)
            # model_px_size = models_list[0][1].size
            # model_real_size = models_list[0][2]
            # px_to_mm = min(model_px_size[0]/model_real_size[0], model_px_size[1]/model_real_size[1])
            # px_to_mm = math.floor(px_to_mm*100) / 100
            
            # it's max accurate value for current render parameters. TODO: find some way to calculate it
            # NOT FROM rendered bitmaps
            px_to_mm = 0.49

            # Resize bitmaps in models_list if needed to meet px_to_mm
            models_list = normalize_bitmaps(models_list, px_to_mm)

            # TODO: as config entry
            # Don't use less than 2. Otherwise really buggy.
            mm_chunk_size = 2
            mm_gap_size = 10
            px_chunk_size = math.floor(px_to_mm*mm_chunk_size)
            # If zero, round up to 1.
            if px_chunk_size < 1:
                px_chunk_size = 1

            models_new_position = Autoarranger(table_size, px_chunk_size, mm_chunk_size, mm_gap_size).add_models_list(models_list)

            scene_root = Application.getInstance().getController().getScene().getRoot()

            grouped_operation = GroupedOperation()

            for node in DepthFirstIterator(scene_root):
                for new_position in models_new_position:
                    if id(node) == new_position[0]:
                        world_position = node.getWorldPosition()
                        bounding_box = node.getBoundingBox()
                        real_x_size = node._aabb._max.x - node._aabb._min.x
                        real_z_size = node._aabb._max.z - node._aabb._min.z
                        x_half_size = real_x_size/2
                        z_half_size = real_z_size/2
                        x_rotate_comp = world_position.x - bounding_box.center.x
                        z_rotate_comp = world_position.z - bounding_box.center.z
                        
                        # New position table exists only when model can be placed on plate, else it's placed beside.
                        if new_position[1]:
                            x_chunk_center = (new_position[1][2] * mm_chunk_size) - real_x_size
                            z_chunk_center = (new_position[1][3] * mm_chunk_size) - real_z_size
                            x_pos = new_position[1][0]*mm_chunk_size - table_width/2 + x_half_size + x_chunk_center + x_rotate_comp
                            z_pos = new_position[1][1]*mm_chunk_size - table_depth/2 + z_half_size + z_chunk_center + z_rotate_comp
                        else:
                            x_pos = table_width/2 * 1.1 + x_half_size + table_margin_size
                            z_pos = -table_depth/2 + z_half_size - table_margin_size

                        y_pos = world_position.y - bounding_box.bottom

                        position = Vector(x_pos, y_pos, z_pos)
                        single_operation = TranslateOperation(node, position, set_position = True)
                        grouped_operation.addOperation(single_operation)
            
            grouped_operation.push()
        else:
            print("No models to arrange")

        return True


def createAutoarrangeTool(engine, script_engine):
    if(AutoarrangeTool.instance==None):
        AutoarrangeTool.instance=AutoarrangeTool()

    return AutoarrangeTool.instance
