# Copyright (c) 2015 Ultimaker B.V.
# Uranium is released under the terms of the AGPLv3 or higher.

from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal,QVariant,pyqtProperty
from PyQt5.QtWidgets import QAction
from PyQt5.QtQml import QQmlListProperty



from ListField import ListField
from PrintersSource import PrintersSource
from TypesSource import TypesSource
from MaterialsSource import MaterialsSource
from QualitiesSource import QualitiesSource
from DurabilitiesSource import DurabilitiesSource


from cura import CameraAnimation

class DataModelProxy(QObject):
    """
    Class visible in QML file as SI.DataModelProxy
    """

    ######################### INIT ###########################################################################
    ##########################################################################################################
    def __init__(self, parent = None):
        super().__init__(parent)
        print("Proxy")

        self._printersList = PrintersSource([
            ListField('3D GENCE ONE',1),
            ListField('3D GENCE INDUSTRY',2)
        ])


        self._typesList = TypesSource([
            ListField('Model + Support',1),
            ListField('Model (Single extruder)',2),
            ListField('Model (Both extruders)',3,False,True),
            ListField('Dual Color',4)
        ])
        self._materialsList = MaterialsSource([
            ListField('3DG ABS + 3DG HIPS',1),
            ListField('3DG PLA + 3DG PVA',2,False,True),
            ListField('3DG PLA + 3DG HIPS',3),
            ListField('VERBATM ABS + 3DG',4),
            ListField('HIPS',5),
            ListField('CUSTOM',0,True)
        ])
        self._qualitiesList = QualitiesSource([
            ListField('NORMAL',1),
            ListField('FINE',2)
        ])
        self._durabilitiesList = DurabilitiesSource([
            ListField('NORMAL',1),
            ListField('ENCHANGED',2),
            ListField('SOLID',3)
        ])

        self._showJumps = False

    printersChanged = pyqtSignal(int)
    typesChanged = pyqtSignal(int)
    materialsChanged = pyqtSignal(int)
    qualitiesChanged = pyqtSignal(int)
    durabilitiesChanged = pyqtSignal(int)


    instance = None


    ######################### TYPES ###########################################################################
    @pyqtProperty(QQmlListProperty)
    def typesList(self):
        """
        :return: Types of Profile
        """
        return QQmlListProperty(ListField, self, self._typesList.getFields())

    @pyqtSlot(QVariant)
    def reloadTypes(self,onTypesReady):
        """
        Allows to update list of types
        :param onTypesReady: QAction called when types are ready
        """
        self._typesList.updateFields()
        onTypesReady.trigger()

    @pyqtSlot(int)
    def selectType(self,id):
        """
        Select one type from list ane emit selected type's id
        :param id:
        :return:
        """
        selected_type = self._typesList.getField(id)
        print(selected_type.name)
        self.typesChanged.emit(selected_type.id)



    ######################### PRINTERS ########################################################################
    @pyqtProperty(QQmlListProperty)
    def printersList(self):
        return QQmlListProperty(ListField, self,self._printersList.getFields())


    @pyqtSlot(QVariant)
    def reloadPrinters(self,onPrintersReady):
        self._printersList.updateFields();
        onPrintersReady.trigger()

    @pyqtSlot(int)
    def selectPrinter(self,id):
        selected_printer = self._printersList.getField(id)
        print(selected_printer.name)
        self.printersChanged.emit(selected_printer.id)

    ######################### MATERIALS #######################################################################

    @pyqtProperty(QQmlListProperty)
    def materialsList(self):
        return QQmlListProperty(ListField, self,self._materialsList.getFields())

    @pyqtSlot(QVariant)
    def reloadMaterials(self,onMaterialsReady):
        self._materialsList.updateFields()
        onMaterialsReady.trigger()

    @pyqtSlot(int)
    def selectMaterial(self,id):
        selected_material = self._materialsList.getField(id)
        print(selected_material.name)
        self.materialsChanged.emit(selected_material.id)

    ######################### QUALITIES #######################################################################

    @pyqtProperty(QQmlListProperty)
    def qualitiesList(self):
        return QQmlListProperty(ListField, self,self._qualitiesList.getFields())

    @pyqtSlot(QVariant)
    def reloadQualities(self,onQualitiesReady):
        self._qualitiesList.updateFields()
        onQualitiesReady.trigger()

    @pyqtSlot(int)
    def selectQuality(self,id):
        selected_quality = self._qualitiesList.getField(id)
        print(selected_quality.name)
        self.qualitiesChanged.emit(selected_quality.id)

    ######################### DURABILITIES #######################################################################

    @pyqtProperty(QQmlListProperty)
    def durabilitiesList(self):
        return QQmlListProperty(ListField, self,self._durabilitiesList.getFields())

    @pyqtSlot(QVariant)
    def reloadDurabilities(self,onDurabilitiesReady):
        self._durabilitiesList.updateFields()
        onDurabilitiesReady.trigger()

    @pyqtSlot(int)
    def selectDurability(self,id):
        selected_durability = self._durabilitiesList.getField(id)
        print(selected_durability.name)
        self.durabilitiesChanged.emit(selected_durability.id)



    ######################### MODEL_PARAMETERS ###################################################################

    @pyqtProperty('bool')
    def showJumps(self):
        return self._showJumps

    @showJumps.setter
    def showJumps(self,value):
        self._showJumps=value

def createDataModelProxy(engine, script_engine):

    if(DataModelProxy.instance==None):
        DataModelProxy.instance=DataModelProxy()

    return DataModelProxy.instance
