
from PyQt5.QtCore import Qt, QCoreApplication, pyqtSlot, pyqtProperty
from UM.Application import Application

from UM.Qt.ListModel import ListModel
from .NozzlesModel import NozzlesModel
from .MaterialsModel import MaterialsModel


class QualitiesModel(ListModel):
    QualityNameRole = Qt.UserRole + 1

    def __init__(self, parent=None):

        if Application.getInstance()._qualities != None:
            raise ValueError("Duplicate singleton creation of QualitiesModel")
        Application.getInstance()._qualities = self

        super().__init__(parent)
        self._printerTree = Application.getInstance()._printerTree
        self.addRoleName(self.QualityNameRole, "name")
        self._update()

    def _update(self):
        items = []
        material = QualitiesModel.getInstance().__Material

        if(not len(material) == 0):
            printer = NozzlesModel.getInstance().getPrinter()
            nozzle = MaterialsModel.getInstance().getNozzle()

            node = self._printerTree.getQualitiesNode(
                printer, nozzle, material)
            nozzles = node.getCurrentNode()[2]

            for i in nozzles:
                item = {"name": i}
                items.append(item)

        self.setItems(items)

    def getMaterial(self):
        return QualitiesModel.getInstance().__Material

    __Material = ""

    @classmethod
    def getInstance(cls):
        if Application.getInstance()._qualities is None:
            Application.getInstance()._qualities = cls()

        return Application.getInstance()._qualities

    @staticmethod
    def onMaterialChanged(newMaterial):
        QualitiesModel.getInstance().__Material = newMaterial
        QualitiesModel.getInstance()._update()


def CreateQualitiesModel(engine, script_engine):
    return QualitiesModel.getInstance()
