from UM.Application import Application
from UM.Settings.Models.SettingPropertyProvider import SettingPropertyProvider

class AdditionsSettingsOverrideManager:
    def __init__(self):
        #self._instance = None
        self._options = {
            'support_angle': None,
            'support_pattern': None,
            'support_line_distance': None,
            'cool_fan_speed': None,
            'xy_offset': None,
            'support_enable': None
        }
    _xy_compensation = None
    _instance = None

    @classmethod
    def GetInstance(cls):
        if AdditionsSettingsOverrideManager._instance is None:
            AdditionsSettingsOverrideManager._instance = cls()

        return AdditionsSettingsOverrideManager._instance

    def SetOption(self, key: str, value: str):
        if key in self._options:
            if value == "None":
                self._options[key] = None
            else:
                self._options[key] = value
            #print(self._options)
            return True
        return False

    def GetOption(self, key):
        if key in self._options:
            if self._options[key] is None:
                return "None"
            else:
                return self._options[key]

        print("Wrong setting!")
        return False

    def IsOverriden(self, key: str):
        if key in self._options:
            if self._options[key] is not None:
                return True
        return False

    def GetOverridenSettings(self):
        # print(self._options)
        # print(self._xy_compensation)
        return {k:v for k,v in self._options.items() if v is not None}

    def InitOptionValue(self, key:str):
        propertyProvider = SettingPropertyProvider()
        stackId = Application.getInstance().getGlobalContainerStack()._id
        propertyProvider.setContainerStackId(stackId)
        propertyProvider.setKey(key)
        result = propertyProvider._getPropertyValue("value")

        return str(result)

    def SetXYCompensation(self, factor: str):
        if factor == 'None':
            self._xy_compensation = None
        else:
            self._xy_compensation = round(float(factor)/100, 4)

    def XYCompensation(self):
        if self._xy_compensation is None:
            return

        scale_tool = Application.getInstance().getController().getTool('GenceScaleTool')
        scale_tool.setCompensationScale(self._xy_compensation)

    def IsXYCompensation(self):
        if self._xy_compensation is None:
            return False

        return True
