from PyQt5.QtCore import QUrl, QObject, pyqtSlot, pyqtSignal, QVariant, pyqtProperty

from cura import CustomSettingsActions
from cura.SIProxies.Models.NozzlesModel import NozzlesModel
from cura.SIProxies.Models.MaterialsModel import MaterialsModel
from cura.SIProxies.Models.QualitiesModel import QualitiesModel
from cura.SIProxies.Models.DurabilitiesModel import DurabilitiesModel

from cura.SIProxies.BaseNotifications import BaseNotifications
from cura.SIProxies.BaseNotifications import JSFunction

from cura.SIProxies.SettingsManage.PrintersTree import PrinterTree
from cura.SIProxies.SettingsManage.ProfileLoaderManager import ProfileLoaderManager
from cura.View.RightMenuManager import RightMenuManager
from cura.SIProxies.SettingsManage.AdditionsSettingsOverrideManager import AdditionsSettingsOverrideManager

# To creat gcode comment and use Scene
from UM.Controller import Controller
from UM.Application import Application

from UM.PluginRegistry import PluginRegistry
from cura.SIProxies.GencePrinterManager import GencePrinterManager
from UM.Resources import Resources


class NotificationsProxy(BaseNotifications):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._tree = PrinterTree.getInstance()

        self.printerChanged.connect(NozzlesModel.onPrinterChanged)
        self.nozzleChanged.connect(MaterialsModel.onNozzleChanged)
        self.materialChanged.connect(QualitiesModel.onMaterialChanged)
        self.qualityChanged.connect(DurabilitiesModel.onQualityChanged)
        self._scene = Application.getInstance().getController().getScene()
        self._commentCreator = PluginRegistry.getInstance().getPluginObject("CommentCreator")

        self.durabilityChanged.connect(PrinterTree.getInstance().onDurabilitySelected)

    def __del__(self):
        self.printerChanged.disconnect(NozzlesModel.onPrinterChanged)
        self.nozzleChanged.disconnect(MaterialsModel.onNozzleChanged)
        self.materialChanged.disconnect(QualitiesModel.onMaterialChanged)
        self.qualityChanged.disconnect(DurabilitiesModel.onQualityChanged)
        self.durabilityChanged.disconnect(
            PrinterTree.getInstance().onDurabilitySelected)

    instance = None
    __Printer = ""
    __Nozzle = ""
    __Material = ""
    __Quality = ""
    __Durability = ""

    __override_profile = None

    printerChanged = pyqtSignal(str, arguments=['printer'])
    nozzleChanged = pyqtSignal(str, arguments=['nozzle'])
    materialChanged = pyqtSignal(str, arguments=['material'])
    qualityChanged = pyqtSignal(str, arguments=['quality'])
    # selected durability and path

    # dur,quali,material,nozzle,printer
    durabilityChanged = pyqtSignal(
        str, str, str, str, str, arguments=['durability'])

    @pyqtProperty(str)
    def support_enable(self):
        return AdditionsSettingsOverrideManager.GetInstance().GetOption(key="support_enable")

    @support_enable.setter
    def support_enable(self, value):
        AdditionsSettingsOverrideManager.GetInstance().SetOption("support_enable", value)

    @pyqtProperty(str)
    def rigid_enable(self):
        value = AdditionsSettingsOverrideManager.GetInstance().GetOption(key="support_pattern")

        if value == 'None' or value is None:
            return "False"
        else:
            return "True"

    @rigid_enable.setter
    def rigid_enable(self, value):
        if value == "True":
            AdditionsSettingsOverrideManager.GetInstance(
            ).SetOption("support_pattern", "triangles")
            AdditionsSettingsOverrideManager.GetInstance(
            ).SetOption("support_line_distance", "7")
        else:
            AdditionsSettingsOverrideManager.GetInstance().SetOption("support_pattern", "None")
            AdditionsSettingsOverrideManager.GetInstance().SetOption(
                "support_line_distance", "None")

    @pyqtSlot(str, str)
    def set_cura_param(self, key: str, value: str):
        AdditionsSettingsOverrideManager.GetInstance().SetOption(key, value)

    @pyqtSlot(str, result=str)
    def init_cura_param(self, key):
        return AdditionsSettingsOverrideManager.GetInstance().InitOptionValue(key)

    @pyqtSlot(str)
    def set_xy_compensation(self, factor):
        AdditionsSettingsOverrideManager.GetInstance().SetXYCompensation(factor)

    @pyqtProperty(bool)
    def is_xy_compensation(self):
        return AdditionsSettingsOverrideManager.GetInstance().IsXYCompensation()

    @pyqtProperty(str, notify=printerChanged)
    def printer(self):
        return self.__Printer

    @printer.setter
    def printer(self, newPrinter: str):
        self.__Printer = newPrinter

        ProfileLoaderManager.GetInstance().special_case(newPrinter)
        self.__Printer = ProfileLoaderManager.GetInstance().get_special_parameter("Printer")
        RightMenuManager.GetInstance().setPrinterMenu(self.__Printer)

        self._commentCreator.updateComment("Machine", self.__Printer)
        self.printerChanged.emit(self.__Printer)
        # self.printerChangedJS.emit(self.getCallback("onPrinterChanged"))

    @pyqtProperty(str, notify=nozzleChanged)
    def nozzle(self):
        return self.__Nozzle

    @nozzle.setter
    def nozzle(self, newNozzle: str):
        self.__Nozzle = newNozzle

        if ProfileLoaderManager.GetInstance().get_special_case():
            self.__Nozzle = ProfileLoaderManager.GetInstance().get_special_parameter("Nozzle")
            RightMenuManager.GetInstance().setNozzleMenu(self.__Nozzle)

        self._commentCreator.updateComment("Nozzle", self.__Nozzle)
        self._commentCreator.updateComment("Module", self.__Nozzle)
        self.nozzleChanged.emit(self.__Nozzle)

    @pyqtProperty(str, notify=materialChanged)
    def material(self):
        return self.__Material

    @material.setter
    def material(self, newmaterial: str):
        self.__Material = newmaterial

        if ProfileLoaderManager.GetInstance().get_special_case():
            self.__Material = ProfileLoaderManager.GetInstance().get_special_parameter("Material")

        RightMenuManager.GetInstance().setMaterialMenu(self.__Material)

        self._commentCreator.updateComment("Material", self.__Material)
        self.materialChanged.emit(self.__Material)

    @pyqtProperty(str, notify=qualityChanged)
    def quality(self):
        return self.__Quality

    @quality.setter
    def quality(self, newquality: str):
        self.__Quality = newquality

        if ProfileLoaderManager.GetInstance().get_special_case():
            self.__Quality = ProfileLoaderManager.GetInstance().get_special_parameter("Quality")
            RightMenuManager.GetInstance().setQualityMenu(self.__Quality)

        self._commentCreator.updateComment("Quality", self.__Quality)
        self.qualityChanged.emit(self.__Quality)

    @pyqtProperty(str, notify=durabilityChanged)
    def durability(self):
        return self.__Durability

    manualProfileLoaded = pyqtSignal()

    @durability.setter
    def durability(self, newdurability: str):
        self.__Durability = newdurability

        if ProfileLoaderManager.GetInstance().get_special_case():
            self.__Durability = ProfileLoaderManager.GetInstance().get_special_parameter("Durability")
            RightMenuManager.GetInstance().setDurabilityMenu(self.__Durability)
            ProfileLoaderManager.GetInstance().set_special_case(False)

        self._commentCreator.updateComment("Durability", self.__Durability)
        self.durabilityChanged.emit(self.__Durability, self.__Quality,
                                    self.__Material, self.__Nozzle, self.__Printer)
        ProfileLoaderManager.GetInstance().set_last_used(self.__Printer, self.__Nozzle,
                                                         self.__Material, self.__Quality, self.__Durability)
        print(self.__Printer, self.__Nozzle, self.__Material,
              self.__Quality, self.__Durability)
        self.manualProfileLoaded.emit()

    def set_gcode_profile(self, printer_name: str = None):
        current_profile = [self.__Printer, self.__Nozzle, self.__Material, self.__Quality, self.__Durability]
        ProfileLoaderManager.GetInstance().set_gcode_profile(current_profile, printer_name)

    @pyqtSlot()
    def unset_gcode_profile(self):
        ProfileLoaderManager.GetInstance().unset_gcode_profile()

    @pyqtSlot(result=QUrl)
    def get_profiles_folder(self):
        path = Resources.getCustomProfilesPath(False)
        qurl = QUrl.fromLocalFile(path)
        return qurl

    @classmethod
    def GetInstance(cls):
        if NotificationsProxy.instance is None:
            return False

        return NotificationsProxy.instance


def createNotificationsProxy(engine, script_engine):

    if(NotificationsProxy.instance == None):
        if(NotificationsProxy.instance == None):
            NotificationsProxy.instance = NotificationsProxy()

    return NotificationsProxy.instance
