from .PrinterSettingsDB import PrinterSettingsDB
import collections


class TreeNode:
    def __init__(self,name:str,depth=0):
        self._dict = collections.OrderedDict()#{}
        self._name = name
        self._depth = depth
        self._version = -1

    def __str__(self):
        return repr(self)

    def __repr__(self):
        'Return the three as text'
        output = ""
        for i in range(self._depth):
                 output+="\t"
        output+= self._name + " " + ": "+ str(self._depth)
        children=""
        for tree in self._dict.values():
             children = children+ " \n" + str(tree)

        output += children
        return output

    def insert(self,key,value):
        self._dict[key] = value

    def getValue(self,key):
        if not key in self._dict.keys():
            return None;
        return self._dict[key]

    def getCurrentNode(self) -> (int, str, [str]):
        'Return current node depth ,name and childrens names'
        array = []
        for key in self._dict.keys():
            array.append(key)
        return  (self._depth,self._name,array)

    def getNextNode(self, name: str):
        if(name in self._dict.keys()):
            return self._dict[name]
        return TreeNode("empty")

    # def getVersion(self) -> int:
    #     return self.version



class TreeGenerator:
    def __init__(self):
        self.database = None

    def __populate_tree(self, tree: TreeNode, conditions: collections.OrderedDict = collections.OrderedDict([('printer', None), ('nozzle', None), ('material', None), ('quality', None), ('durability', None)]), depth=0):
        leafs = self.database.get_elements(conditions)

        depth = depth + 1

        if leafs != "empty":
            new_conditions = collections.OrderedDict(conditions)
            column_name = leafs[0]

            for value in leafs[1]:
                subTree = TreeNode(value[0], depth)
                tree.insert(value[0], subTree)
                if(column_name != "durability"):
                    new_conditions[column_name] = value[0]
                    self.__populate_tree(subTree, new_conditions, depth)

    def CreateTreeFromDirectories(self)->TreeNode:
        self.database = PrinterSettingsDB()
        tree = TreeNode(name="Settings")
        self.__populate_tree(tree)

        self.database.close()

        return tree
