from UM.Math.Quaternion import Quaternion
from UM.Operations.Operation import Operation
from UM.Signal import Signal, signalemitter
import math

@signalemitter
class GenceOrientFaceToPlaneOperation(Operation):
    propertyChanged = Signal()

    def __init__(self, node, quaternion_rotation):
        super().__init__()
        self._node = node
        self._quaterion = quaternion_rotation

        self._old_rotation = None
        self._original_position = node.getPosition()

        x, y, z = self._quaternionToEulerianAngle(quaternion_rotation.x,
                                                  quaternion_rotation.y,
                                                  quaternion_rotation.z,
                                                  quaternion_rotation.w)

        self._x = x % 360
        self._y = y % 360
        self._z = z % 360

        self._px = getattr(node, "_rotationX", 0)
        self._py = getattr(node, "_rotationY", 0)
        self._pz = getattr(node, "_rotationZ", 0)

    def undo(self):
        if self._old_rotation:
            self._node.setOrientation(self._old_rotation)
        else:
            self._node.setOrientation(Quaternion())

        self._node._place_rotation = self._old_rotation
        self._node.setPosition(self._original_position)

        self._node._rotationX = self._px
        self._node._rotationY = self._py
        self._node._rotationZ = self._pz

        self.propertyChanged.emit()

    def redo(self):
        self._node.setOrientation(self._quaterion)
        self._old_rotation = getattr(self._node, "_plane_rotation", False)
        self._node._plane_rotation = self._quaterion

        setattr(self._node, "_rotationX", self._x)
        setattr(self._node, "_rotationY", self._y)
        setattr(self._node, "_rotationZ", self._z)

        self.propertyChanged.emit()

    def _quaternionToEulerianAngle(self, x, y, z, w):
        ysqr = y * y

        t0 = +2.0 * (w * x + y * z)
        t1 = +1.0 - 2.0 * (x * x + ysqr)
        X = math.degrees(math.atan2(t0, t1))

        t2 = +2.0 * (w * y - z * x)
        t2 = 1 if t2 > 1 else t2
        t2 = -1 if t2 < -1 else t2
        Y = math.degrees(math.asin(t2))

        t3 = +2.0 * (w * z + x * y)
        t4 = +1.0 - 2.0 * (ysqr + z * z)
        Z = math.degrees(math.atan2(t3, t4))

        return X, Y, Z
