from UM.Math.Matrix import Matrix
from UM.Math.Vector import Vector
import numpy

# https://math.stackexchange.com/a/476311
# https://en.wikipedia.org/wiki/Rodrigues'_rotation_formula
def CalculateRotationToParallel(a, b):
    if a == b:
        return Matrix([
            [1.0, 0.0, 0.0],
            [0.0, 1.0, 0.0],
            [0.0, 0.0, 1.0]
        ])
    if a == -b:
        # 180 about any axis
        return Matrix([
            [1.0, 0.0, 0.0, 0.0],
            [0.0,-1.0, 0.0, 0.0],
            [0.0, 0.0,-1.0, 0.0],
            [0.0, 0.0, 0.0, 1.0],
        ])

    c = a.dot(b)

    if c < 0:
        c, b = -c, -b

    v = a.cross(b)

    v_x = Matrix([
        [0.0, -v.z, v.y],
        [v.z, 0.0, -v.x],
        [-v.y, v.x, 0.0]
    ])

    I = Matrix([
        [1.0, 0.0, 0.0],
        [0.0, 1.0, 0.0],
        [0.0, 0.0, 1.0]
    ])

    r1 = AddMatrices(I, v_x)
    r2 = v_x.multiply(v_x, True)
    r2 = MultiplyMatrixByNumber(r2, (1 / (1 + c)))
    R = AddMatrices(r1, r2)

    return R


def CalculatePolygonNormal(vertex_1, vertex_2, vertex_3):
    u = vertex_2 - vertex_1
    v = vertex_3 - vertex_1

    x = (u.y * v.z) - (u.z * v.y)
    y = (u.z * v.x) - (u.x * v.z)
    z = (u.x * v.y) - (u.y * v.x)

    return Vector(x, y, z).normalized()


def CalculateCentroid(vert_1, vert_2, vert_3):
    centroid_x = (vert_1.x + vert_2.x + vert_3.x) / 3
    centroid_y = (vert_1.y + vert_2.y + vert_3.y) / 3
    centroid_z = (vert_1.z + vert_2.z + vert_3.z) / 3

    return Vector(x=centroid_x, y=centroid_y, z=centroid_z)

def AddMatrices(matrix_1, matrix_2):
    return Matrix(data=numpy.add(matrix_1.getData(), matrix_2.getData()))

def MultiplyMatrixByNumber(matrix, number):
    return Matrix(data=numpy.multiply(matrix.getData(), number))
