import numpy as np
from UM.Application import Application
from UM.Scene.Selection import Selection
from .SplitMesh import SplitMesh
from .Split import Axis, Side

SPLIT_DEBUG = False


def SimpleSplitMesh(axis, value):
    print("SimpleSplitMesh {{axis : {}, cut_value : {}}}".format(str(axis), str(value)))
    job = SimpleSplitMeshJob(axis, value)
    job.run() if SPLIT_DEBUG else job.start()


class SimpleSplitMeshJob(SplitMesh):
    def __init__(self, axis, value):
        value = np.float32(value)
        super().__init__(axis, value, SPLIT_DEBUG)

        self.__axis = axis

    def run(self):
        node = Selection.getCurrentlySelectedObjects()[0]
        mesh = node.getMeshDataTransformed()
        vertices = mesh.getVertices()
        normals = mesh.getNormals()

        self._initialize_progress(1, "Splitting...")

        cut_halfs = self._calculate_cut_halfs(vertices, normals, 0.9)

        left_vertices = cut_halfs['left_vertices']
        right_vertices = cut_halfs['right_vertices']
        left_segments_normals = cut_halfs['left_segments_normals']
        right_segments_normals = cut_halfs['right_segments_normals']
        left_sections_on_cut = cut_halfs['left_sections_on_cut']
        right_sections_on_cut = cut_halfs['right_sections_on_cut']

        self._debug_drawSections(left_sections_on_cut)
        self._debug_drawSections(right_sections_on_cut)

        if left_vertices and right_vertices:
            closing_left_polygons = []
            closing_right_polygons = []

            # Left
            if left_sections_on_cut:
                closing_left_polygons = self._prepare3dPolygons(left_sections_on_cut, left_segments_normals)
            self._add_node_progress(0.04)

            # Right
            if right_sections_on_cut:
                closing_right_polygons = self._prepare3dPolygons(right_sections_on_cut, right_segments_normals)
            self._add_node_progress(0.04)

            # Small adjustments
            if self.__axis == Axis.X or self.__axis == Axis.Z:
                if closing_right_polygons:
                    closing_right_polygons.reverse()
            if self.__axis == Axis.Y:
                if closing_left_polygons:
                    closing_left_polygons.reverse()

            # Add prepared polygons to models
            left_vertices.extend(closing_left_polygons)
            right_vertices.extend(closing_right_polygons)

            self._swap_nodes_operation(node, left_vertices, right_vertices)

            self._add_node_progress(0.02)
            self._display_status("Model split successfully")
        else:
            Application.getInstance().displayStatus("Nothing to split.")

        self._finish_progress()
