import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../singletons"
import "../tweaks"
import "../spinBox"

Item {
    id: button

    property alias buttonText: buttonText.text
    property alias buttonTextVisibility: buttonText.visible
    property alias buttonTextBackgroundHeight: textBackground.height
    property alias buttonWidth: button.width
    property alias buttonBackgroundColor: buttonBackground.color
    property alias deactivateButton: deactivateIndicator.visible

    property alias auto: autoText.visible
    property alias spinBox: spinBoxItem.visible
    property alias spinBoxUnit: spinBox.unitString

    property alias minValue: spinBox.minValue
    property alias maxValue: spinBox.maxValue
    property alias singleStep: spinBox.singleStep
    property alias acceptDot: spinBox.acceptDot
    property alias precision: spinBox.precision
    property alias curaParam: spinBox.curaParam

    property alias autoMouseArea: autoMouseArea
    property alias undoMouseArea: undoMouseArea

    property string objectHelpText: ""
    property string objectHelpImageSource: ""


     ///temporary alias
    property alias spinBoxAlias: spinBox

    Layout.preferredHeight: Globals.buttonHeight
    Layout.fillWidth: true

    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: 0

        Rectangle {
            id: textBackground
            Layout.preferredHeight: Globals.buttonHeaderHeight
            Layout.fillWidth: true
            color: Colors.menuButton

            Text {
                id: buttonText
                anchors.centerIn: parent
                font.pixelSize: Globals.fontPixelSize
                font.capitalization: Font.AllUppercase
                font.family: Fonts.boxed

                color: Colors.buttonText
                text: "button"
            }
        }

        Rectangle {
            id: decorator
            anchors.top: textBackground.bottom
            Layout.fillWidth: true
            Layout.preferredHeight: Globals.decoratorHeight
            color: Colors.buttonDecorator
        }

        Rectangle {
            id: buttonBackground
            Layout.fillHeight: true
            Layout.fillWidth: true
            color: Colors.buttonBackground

            Item{
                anchors.fill: parent

                Text {
                    id: autoText
                    anchors.centerIn: parent
                    text: "Auto"
                    font.pixelSize: Globals.fontPixelSize
                    font.family: Fonts.boxed
                    color: Colors.buttonOptionText
                }

                MouseArea {
                    id: autoMouseArea
                    visible: autoText.visible
                    anchors.fill: parent
                }
            }

            RowLayout{
                id: spinBoxItem
                anchors.fill: parent
                anchors.leftMargin: (parent.height - spinBox.height)/2
                anchors.rightMargin: (parent.height - spinBox.height)/2
                spacing: parent.width * 0.05
                visible: false
                CustomSpinBox {
                    id: spinBox
                    standalone: true
                }

                Image {
                    Layout.preferredHeight: parent.height * 0.5
                    Layout.preferredWidth: height

                    fillMode: Image.PreserveAspectFit
                    mipmap: true

                    source: "../images/undo.svg"

                    MouseArea {
                        id: undoMouseArea
                        anchors.fill: parent
                    }
                }
            }
        }
    }

    //Non-active button layer
    Rectangle {
        id: deactivateIndicator
        anchors.fill: parent
        visible: false
        color: Colors.buttonDeactivator
    }

    BottomShadow {}

    ButtonTweaks {
        id: buttonTweaks
        z: -1
        hoverIndicatorEnabled: false
        helpText: objectHelpText
        helpImageSource: objectHelpImageSource
    }
}
