import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 1.4

import "../singletons"
import "../tweaks"

Item {
    property alias menuButtonText: text.text
    property alias menuButtonMouseArea: mouseArea
    property alias buttonClickedAction: buttonClicked
    property alias helpText: buttonTweaks.helpText
    property alias helpImageSource: buttonTweaks.helpImageSource
    property alias buttonAction: mouseArea
    property alias deactivate: deactivateIndicator.visible
    property string buttonHelpText: ""
    property real heightFactor: 2
    property bool shadowEnabled: true

    width: parent.width
    height: text.height * heightFactor

    Layout.fillWidth: true
    // TODO: probably should be text.height * 2.5. To check using final font
    Layout.preferredHeight: height

    Action
    {
        id:buttonClicked
    }

    Rectangle {
        anchors.fill: parent
        color: Colors.menuButton

        Text {
            id: text
            color: Colors.buttonText
            anchors.centerIn: parent
            font.pixelSize: Globals.fontPixelSize
            font.family: Fonts.boxed
            text: "baton"
        }

        MouseArea {
            id: mouseArea
            visible: !deactivate
            anchors.fill: parent

            onPressed: onPressedAnimation.start()
            onReleased: onReleasedAnimation.start()

            onClicked: buttonClicked.trigger()
        }

        Rectangle {
            id: clickIndicator
            anchors.fill: parent
            visible: !deactivate

            color: Colors.buttonClicked

            opacity: 0

            PropertyAnimation on opacity {
                id: onPressedAnimation
                easing.type: Easing.OutExpo
                duration: 80
                to: 0.6
                running: false
            }

            PropertyAnimation on opacity {
                id: onReleasedAnimation
                duration: 80
                to: 0.0
                running: false
            }
        }

        Rectangle {
            id: deactivateIndicator
            anchors.fill: parent
            visible: false
            color: Colors.buttonDeactivator
        }
    }

    ButtonTweaks {
        id: buttonTweaks
        hoverIndicatorEnabled: !deactivate
        helpText: buttonHelpText
    }

    FullShadow {
        visible: shadowEnabled
    }
}
