import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../singletons"
import "../tweaks"
import "menu"

Item {
    id: button

    property alias buttonText: buttonNameText.text
    property alias buttonTextVisibility: buttonNameText.visible
    property alias buttonIcon: buttonIcon.source
    property alias buttonIconWhite: buttonIconWhite.source
    property alias buttonTextBackgroundHeight: textBackground.height
    property alias buttonAction: mouseArea
    property alias buttonWidth: button.width
    property alias menu: menu
    property alias menuOpened: menu.visible
    property alias buttonClicked: buttonIconWhite.visible
    property alias deactivateButton: deactivateIndicator.visible
    property alias cornerTriangleInColor: corner.visible
    property alias content: menu.contentLoader

    property alias helpText: buttonTweaks.helpText
    property alias menuHelpText: menu.helpText
    property alias menuHelpImageSource: menu.helpImageSource

    property color buttonBackgroundColor: Colors.buttonBackground

    property color buttonOptionTextColor: Colors.buttonOptionText

    property bool replacePlusChar: false

    property string currentOption

    Layout.preferredHeight: Globals.buttonHeight
    Layout.fillWidth: true

    MouseArea {
        id: mouseArea
        visible: !deactivateIndicator.visible
        anchors.fill: parent
    }

    RowLayout{
        anchors.fill: parent
        spacing: 0

        ColumnLayout {
            anchors.fill: parent
            spacing: 0

            Rectangle {
                id: textBackground
                Layout.preferredHeight: Globals.buttonHeaderHeight 
                Layout.fillWidth: true
                color: Colors.buttonTextBackground

                Text {
                    id: buttonNameText
                    x: parent.height - Globals.fontPixelSize
                    anchors.verticalCenter: parent.verticalCenter
                    font.pixelSize: Globals.fontPixelSize
                    font.capitalization: Font.AllUppercase
                    font.family: Fonts.boxed

                    color: Colors.buttonText
                    text: "button"
                }
            }

            Rectangle {
                id: decorator
                Layout.fillWidth: true
                Layout.preferredHeight: Globals.decoratorHeight
                color: Colors.buttonDecorator
            }

            Rectangle {
                id: buttonBackground
                Layout.fillHeight: true
                Layout.fillWidth: true
                color: buttonBackgroundColor

                Text {
                    id: activeOption
                    x: parent.width * 0.1
                    text: replacePlusChar ? currentOption.replace("+ ", "\n") : currentOption
                    anchors.verticalCenter: parent.verticalCenter
                    font.pixelSize: Globals.fontPixelSize
                    font.family: Fonts.boxed
                    color: buttonOptionTextColor
                    width: parent.width * 0.85
                    wrapMode: Text.Wrap
                }

                //Corner triangle
                Image {
                    id: corner
                    visible: true
                    height: Globals.buttonTriangleSize
                    y: (parent.height * 0.93) - corner.height
                    x: parent.height - y - height

                    mirror: true
                    fillMode: Image.PreserveAspectFit
                    source: "../images/corner.svg"
                }

                //Corner triangle white
                Image {
                    id: cornerWhite
                    visible: !(corner.visible)
                    y: (parent.height * 0.93) - corner.height
                    x: parent.height - y - height
                    height: Globals.buttonTriangleSize

                    mirror: true
                    fillMode: Image.PreserveAspectFit
                    source: "../images/corner_white.svg"
                }
            }
        }

        Rectangle {
            Layout.preferredHeight: parent.height
            Layout.preferredWidth: parent.height
            color: buttonBackgroundColor

            //button icon grey
            Image {
                id: buttonIcon
                visible: !(buttonIconWhite.visible)
                anchors.centerIn: parent
                height: Globals.rightMenuButtonIconSize
                mipmap: true

                fillMode: Image.PreserveAspectFit

                source: "../images/placeholder.svg"
            }

            //button icon white
            Image {
                id: buttonIconWhite
                visible: false
                anchors.centerIn: parent
                height: Globals.rightMenuButtonIconSize
                mipmap: true

                fillMode: Image.PreserveAspectFit

                source: "../images/placeholder_white.svg"
            }
        }
    }

    //TODO: probably not necessary here
    //Non-active button layer
    Rectangle {
        id: deactivateIndicator
        anchors.fill: parent
        visible: false
        color: Colors.buttonDeactivator
    }

    BottomShadow {}

    ButtonTweaks{
        id: buttonTweaks
        hoverIndicatorEnabled: !deactivateIndicator.visible && !menuOpened
        helpEnabled: !deactivateIndicator.visible
    }

    ButtonMenu {
        id: menu
        autoclose: true
        popup.x: -(popup.width)
    }
}
