import QtQuick 2.0
import QtQuick.Layouts 1.1
import QtQuick.Controls 2.0

import Gence 1.0 as Gence
import Cura 1.0 as Cura
import UM 1.3 as UM

import "../../tweaks"
import "../../buttons"
import "../../singletons"

Item {
    property alias layoutHeight: layout.height
    property string listHelpText: "Click to choose model."
    property bool multipleExtrudersProfile: machineExtruderCount.properties.value > 1

    ColumnLayout {
        id: layout
        spacing: parent.width * 0.04
        anchors.fill: parent
        Rectangle {
            Layout.fillWidth: true
            Layout.preferredHeight: Globals.fontPixelSize * 1.5 * 10 + border.width * 2
            // Layout.fillHeight: true
            color: Colors.customListBackground
            border.width: 1
            border.color: Colors.customListBorder
            clip: true
            Row {
                anchors.margins: parent.border.width
                anchors.fill: parent
                ListView {
                    id: list
                    width: parent.width - scroll.width * scroll.visible
                    height: parent.height
                    clip: true
                    boundsBehavior: Flickable.StopAtBounds
                    model: Gence.ModelsHolder.printableNodes
                    ScrollBar.vertical: scroll
                    delegate: ModelsListModelButton {}

                    ButtonTweaks {
                        z: -1
                        hoverIndicatorEnabled: false
                        helpText: listHelpText
                    }
                }
                ScrollBar {
                    id: scroll
                    height: parent.height
                    visible: list.contentHeight > list.height ? true : false
                    contentItem: Rectangle {
                        implicitWidth: layout.width * 0.02
                        color: Colors.customListScrollBarHandle
                    }
                    background: Rectangle {
                        anchors.fill: parent
                        color: Colors.customListBackground
                    }
                    
                    ButtonTweaks {
                        hoverIndicatorEnabled: false
                        helpText: listHelpText
                    }
                }
            }
        }

        MenuButton {
            id: addModel
            menuButtonText: "Add new model"
            buttonAction.onClicked: openDialog.open()
        }

        MenuButton {
            id: groupModels
            deactivate: !UM.Selection.hasSelection
            menuButtonText: "Delete model"
            buttonAction.onClicked: Cura.Actions.deleteSelectedObject.trigger()
        }

        MenuButton {
            id: saveAsSTL
            deactivate: !UM.Selection.hasSelection
            menuButtonText: "Save as .stl"
            buttonAction.onClicked: UM.OutputDeviceManager.requestWriteSelectionToDevice("local_file", PrintInformation.jobName, { "filter_by_machine": false, "limit_mimetypes": ['application/x-stl-ascii', 'application/x-stl-binary'] })
        }
    }

    UM.SettingPropertyProvider
    {
        id: machineExtruderCount

        containerStackId: Cura.MachineManager.activeMachineId
        key: "machine_extruder_count"
        watchedProperties: [ "value" ]
        storeIndex: 0
    }
}
