import QtQuick 2.0
import QtQuick.Layouts 1.1

import "../../spinBox"
import "../../tweaks"
import "../../buttons"

Item {
    property alias layoutHeight: layout.height
    property alias xSpinBox: xSpinBox
    property alias ySpinBox: ySpinBox
    property alias zSpinBox: zSpinBox
    property alias placeButtonAction: placeButton.menuButtonMouseArea
    property alias resetMoveButtonAction: resetMoveButton.menuButtonMouseArea
    property alias x_precision: xSpinBox.precision
    property alias y_precision: ySpinBox.precision
    property alias z_precision: zSpinBox.precision

    ColumnLayout {
        id: layout
        spacing: height * 0.03
        anchors.fill: parent
        LabeledSpinBox {
            id: xSpinBox
            spinBoxLabel: "X:"
            spinBoxUnit: " mm"
            acceptDot: true
            instantAccept: false
            precision: 2
            Layout.bottomMargin: 0

            property string objectHelpText: "Move the model in X axis by 1 mm (each press when using the buttons) " +
                      "or any distance when you enter the desired value in the text field"
            property url objectHelpImageSource: "../../images/help/MoveX.png"
        }
        LabeledSpinBox {
            id: ySpinBox
            spinBoxLabel: "Y:"
            spinBoxUnit: " mm"
            acceptDot: true
            instantAccept: false
            precision: 2
            Layout.bottomMargin: 0

            property string objectHelpText: "Move the model in Y axis by 1 mm (each press when using the buttons) " +
                      "or any distance when you enter the desired value in the text field"
            property url objectHelpImageSource: "../../images/help/MoveY.png"
        }
        LabeledSpinBox {
            id: zSpinBox
            spinBoxLabel: "Z:"
            spinBoxUnit: " mm"
            acceptDot: true
            instantAccept: false
            precision: 2
            Layout.bottomMargin: 0

            property string objectHelpText: "Move the model in Z axis by 1 mm (each press when using the buttons) " +
                      "or any distance when you enter the desired value in the text field"
            property url objectHelpImageSource: "../../images/help/MoveZ.png"
        }

        MenuButton {
            id: placeButton
            menuButtonText: "Place"

            helpText: "Change position in Z axis to place the model on the buildplate"
        }

        MenuButton {
            id: resetMoveButton
            menuButtonText: "Reset"

            helpText: "Restore the initial model position values used when loading .stl file"
        }
    }
}
