import QtQuick 2.4
import QtQuick.Layouts 1.1
import QtQuick.Controls.Styles 1.4

import UM 1.2 as UM

import "../../buttons"
import "../../tweaks"
import "../../spinBox"
import "../../slider"
import "../../singletons"
import "../../common"

Item {
    property alias spinBox: spinBox
    property alias slider: slider 
    property alias axisButton: axisButton
    property alias cutTypeButton: cutTypeButton
    property string cutTypeString: "Simple"
    property string axisString: "X"
    property alias edgeSpinBox: edgeSpinBox
    property alias angleSpinBox: angleSpinBox
    property alias offsetSpinBox: offsetSpinBox
    property alias lengthSpinBox: lengthSpinBox
    property alias invertCheckBox: invertCheckBox
    property bool enableUI: true

    property double step: 0.1

    ColumnLayout {
        id: layout
        anchors.fill: parent
        spacing: parent.width * 0.05

        MenuButton {
            id: cutTypeButton
            menuButtonText: "Cut type: " + cutTypeString
            helpText: objectHelpText
            helpImageSource: objectHelpImageSource
        }

        MenuButton {
            id: axisButton
            menuButtonText: "Axis: " + axisString
            deactivate: false
            helpText: objectHelpText
            helpImageSource: objectHelpImageSource
        }

        Spacer {}

        CustomSpinBox {
            id: spinBox
            acceptDot: true
            standalone: true
            unitString: " mm"
            minValue: 0
            valueText: "0"
            maxValue: 100
            singleStep: 1
            precision: 2
            deactivate: false
        }

        HorizontalSlider {
            id: slider
            minimumValue: spinBox.minValue
            maximumValue: spinBox.maxValue
            deactivate: false
        }

        Spacer {}

        LabeledSpinBox {
            id: edgeSpinBox
            visible: advancedMode
            spinBoxLabel: "Edge"
            labelWidthRatio: 0.25
            acceptDot: true
            standalone: true
            unitString: " mm"
            minValue: 0
            maxValue: 8
            singleStep: step
            roundToStep: true
            precision: 2
        }

        LabeledSpinBox {
            id: angleSpinBox
            visible: advancedMode
            spinBoxLabel: "Angle"
            labelWidthRatio: 0.25
            acceptDot: false
            standalone: true
            unitString: "°"
            minValue: 10
            maxValue: 90
            singleStep: 5
        }

        LabeledSpinBox {
            id: offsetSpinBox
            visible: advancedMode
            spinBoxLabel: "Offset"
            labelWidthRatio: 0.25
            acceptDot: true
            standalone: true
            unitString: " mm"
            minValue: 0
            maxValue: 3
            singleStep: step
            roundToStep: true
            precision: 2
        }

        LabeledSpinBox {
            id: lengthSpinBox
            visible: advancedMode
            spinBoxLabel: "Depth"
            labelWidthRatio: 0.25
            acceptDot: true
            standalone: true
            unitString: " mm"
            minValue: 0.5
            maxValue: 30
            singleStep: step
            roundToStep: true
            precision: 2
        }

        LabeledCheckBox {
            id: invertCheckBox
            visible: advancedMode
            labelText: "Invert"
        }

        Spacer {}

        MenuButton {
            id: splitButton
            menuButtonText: "Split Model"
            buttonClickedAction.onTriggered:{
                enableUI = false
                UM.ActiveTool.triggerAction("runFunction")
            }
            deactivate: !enableUI
            helpText: objectHelpText
            helpImageSource: objectHelpImageSource
        }
    }
}
