import QtQuick 2.4
import QtQuick.Layouts 1.1

import "../../singletons"
import "../../cameraCube"
import "../../slider"
import "../../buttons"
import "../../tweaks"

import SI 1.0 as SI
import Gence 1.0 as Gence

Item {

    property bool  sliderVisible: false
    property alias textLabel:     textLabel
    property alias layerSlider:   layerSlider
    property alias rightLayout:   rightLayout

    property string layerViewHelpText: "Preview of the printout in the form of paths, " +
        "arranged in the 3D printing process for you particular model. The preview presents " +
        "the entire model as a mesh and the last 3 layers as the simulated tool paths. Using " +
        "the active layer change option you can preview earlier model layers.\n\n" +

        "Grey — the Model\n" +
        "Blue and black — Supports\n" +
        "Red — preview of selected layers"

    id: viewButtonUI

    Connections
    {
        target:Gence.Actions.sliced

        onTriggered: {
            sliderVisible = true
        }

    }

    Connections
    {
        target:Gence.Actions.gcode_loaded

        onTriggered: {
            sliderVisible = true
        }

    }
    Connections
    {
        target:Gence.Actions.node_loaded

        onTriggered: {
            sliderVisible = false
        }

    }

    RowLayout {

        id:           layout
        anchors.fill: parent

        ColumnLayout {


            id:    leftLayout
            width: viewButtonUI.width * 0.8

            Text {
                id:                  text
                text:                "Choose wall or corner"
                color:               Colors.cubeDescription
                font.pixelSize:      Globals.fontPixelSize
                font.family:         Fonts.boxed
                horizontalAlignment: Text.Center
                Layout.fillWidth:    true
            }

            CameraCube {}

            MenuButton {
                id:                              centerButton
                menuButtonText:                  "Center"
                buttonClickedAction.onTriggered: SI.CameraOperator.center ()
                helpText: "Center camera on the selected model"
            }

            MenuButton {
                id: perspectiveOffButton
                menuButtonText: "Perspective Off"

                buttonClickedAction.onTriggered:
                {
                    var state = Gence.Notifications.isPerspective

                    if(state) {
                        menuButtonText = "Perspective On"
                    } else {
                        menuButtonText = "Perspective Off"
                    }

                    Gence.Notifications.isPerspective = !state
                }
                helpText: "Switch the perspective on and off"
            }

            MenuButton {
                id:                              resetViewButton
                menuButtonText:                  "Reset View"
                buttonClickedAction.onTriggered: SI.CameraOperator.reset ()
                helpText: "Restore the default camera settings"
            }


            Item {

                visible:                sliderVisible
                Layout.preferredHeight: viewButtonUI.height * 0.06

                Rectangle {

                    x:             0
                    y:             0
                    implicitWidth: 0.5 * leftLayout.width

                    Text {
                        text:                'Layer:'
                        color:               Colors.listText
                        font.pixelSize:      Globals.fontPixelSize
                        font.family:         Fonts.boxed
                        horizontalAlignment: Text.AlignLeft
                        anchors.fill:        parent
                    }

                }

                Rectangle {

                    id:             textField
                    x:              0.5 * leftLayout.width + 1
                    y:              0
                    implicitWidth:  0.5 * leftLayout.width - 2
                    implicitHeight: viewButtonUI.height * 0.06
                    color:          Colors.sliderLayerTextBackground

                    Text {
                        id:                  textLabel
                        color:               Colors.sliderLayerText
                        font.pixelSize:      Globals.fontPixelSize
                        font.family:         Fonts.boxed
                        horizontalAlignment: Text.AlignHCenter
                        anchors.fill:        parent
                    }

                    ButtonTweaks{
                        hoverIndicatorEnabled: false
                        helpText: layerViewHelpText
                    }

                }

            } // end of bottom text field with label


        } // end of left column

        ColumnLayout {

            id:      rightLayout
            width:   viewButtonUI.width * 0.1
            visible: sliderVisible

            LayerSlider {
                id:                             layerSlider
                sliderWithButtons.sliderWidth:  rightLayout.width
                sliderWithButtons.sliderHeight: rightLayout.height
            }

            ButtonTweaks{
                hoverIndicatorEnabled: false
                helpText: layerViewHelpText
            }

        } // end of right column


    } // main row layout end

} // Item end
