import QtQuick 2.5
import QtQuick.Layouts 1.1
import Gence 1.0 as Gence

import "../layout"
import "../layout/menus"
import "../singletons"
import "../modalWindow"

import UM 1.3 as UM

Item {
    anchors.fill: parent
    focus: enabled

    Keys.onPressed:
    {
        event.accepted=false;
    }

    // Turns off dragging and dropping model during and after slicing
    Connections
    {
        target: Gence.Notifications
        onAppStateChanged:
        {
            if(Gence.Notifications.appState === Gence.Notifications.SLICING || Gence.Notifications.appState === Gence.Notifications.GCODE_LOADING)
            {
                dropArea.visible = false
            }
        }
    }

    Component.onCompleted: {
        Globals.modalWindow = modalWindow
    }

    // Turns on dragging and dropping model when slicing was undone
    Connections{
        target: Gence.Actions.abnormalUndo
        onTriggered:
        {
            dropArea.visible = true
        }
    }

    Connections {
        target: Gence.Actions.modalWindowOpened
        onTriggered: {
            deactivateAll.visible = true
            deactivationAnimation.start()
        }
    }

    Connections {
        target: modalWindow
        onClosing: activationAnimation.start()
    }

    Connections {
        target: activationAnimation
        onStopped: {
            deactivateAll.visible = false
        }
    }

    DropArea
    {
        id: dropArea
        anchors.fill: parent;
        onDropped:
        {
            if(drop.urls.length > 0)
            {
                // Import models
                for(var i in drop.urls)
                {
                    // There is no endsWith in this version of JS...
                    if ((drop.urls[i].length <= 12) || (drop.urls[i].substring(drop.urls[i].length-12) !== ".curaprofile")) {
                        // Drop an object
                        //UM.MeshFileHandler.readLocalFile(drop.urls[i]);
                        Printer.readLocalFile(drop.urls[i]);
                    }
                }
            }
        }
    }

    RowLayout
    {
        anchors.fill: parent
        spacing: 0

        LeftMenu
        {
            id:left
            anchors.left: parent.left
            anchors.right: center.left
        }

        CentralArea
        {
            id: center
        }

        RightMenu
        {
            id:right
            anchors.left : center.right
            anchors.right: parent.right
        }
    }

    ModalWindow {
        id: modalWindow
        // anchors.centerIn: parent
        z: 2
    }

    Rectangle {
        id: deactivateAll
        anchors.fill: parent
        visible: false
        color: Colors.buttonDeactivator
        focus: visible

        // Suppress keyboard events (Actions are not included)
        Keys.onPressed: {
            event.accepted = true
        }
        // Suppress mouse events
        MouseArea {
            anchors.fill: parent
            acceptedButtons: Qt.AllButtons
            hoverEnabled: true
            onWheel: { wheel.accepted = true }
        }
    }

    ColorAnimation {
        id: deactivationAnimation
        target: deactivateAll
        property: "color"
        easing.type: Easing.InCubic
        duration: 100
        from: "#00FFFFFF"
        to: Colors.buttonDeactivator
        running: false
    }

    ColorAnimation {
        id: activationAnimation
        target: deactivateAll
        property: "color"
        easing.type: Easing.OutCubic
        duration: 100
        from: Colors.buttonDeactivator
        to: "#00FFFFFF"
        running: false
    }
}
