import QtQuick 2.0
import QtQuick.Layouts 1.1
import UM 1.3 as UM
import Cura 1.0 as Cura
import Gence 1.0 as Gence
import QtQuick.Controls 1.4

import "../../dialogs"
import "../../tweaks"
import "../../buttons"
import "../../singletons"

Item {
    property int appState: Gence.Notifications.appState
    property bool ableToUndoNormal: appState === Gence.Notifications.NODE_LOADED && UM.OperationStack.canUndo
    property bool ableToUndoAbnormal: appState === Gence.Notifications.SLICED || appState === Gence.Notifications.GCODE_LOADED
    property bool modelsListOpened: true
    property bool isOrientFaceToBedActive: false

    property alias openDialog: openDialog

    Component.onCompleted:{
        Gence.Actions.empty_plate.trigger()
    }

    function disableAll()
    {
        Gence.Actions.activeAllTools.enabled=false
        parent.enabled=false
        closeModelButtons()
        deactivateModelButtons()
        addModel.markButtonAsNotActive()
        modelsListButton.markButtonAsNotActive()
        arrangeButton.markButtonAsNotActive()
        duplicateButton.markButtonAsNotActive()
        viewButton.markButtonAsNotActive()
        Gence.Actions.activeAllTools.enabled=false
    }

    Connections
    {
        target: Cura.Actions
        onToggleOrientFaceToPlane:{
            if(!isOrientFaceToBedActive){
                if(!arrangeButton.deactivateButton && Gence.Notifications.NODE_LOADED) //Probably not the best way to determine whether we can use this tool now
                {
                    arrangeButtonAction()
                    UM.Controller.setActiveTool("GenceOrientFaceToPlaneTool")

                    isOrientFaceToBedActive = true
                }
            }
        }
    }

    Connections
    {
        target: Gence.Notifications
        onDeactivatePlaceOnBed: deactivatePlaceOnBedTool(activateMoveTool)
    }

    function deactivatePlaceOnBedTool(activateMoveTool){
        arrangeButton.closeMenu()

        if(activateMoveTool){
            if(UM.Scene.numObjectsSelected != 1){
                UM.Controller.setActiveTool("EmptyTool")
                modelsListOpened = true
                activateModelsOrMove()
            } else {
                UM.Controller.setActiveTool("SITranslateTool")
            }
        }

        isOrientFaceToBedActive = false
    }

    Connections
    {
        target:Gence.Actions.empty_plate
        onTriggered:
        {
            // it's set here due to requirement for opening modelsList when any node(s) is/are loaded
            modelsListOpened = true
            setModelsListVisiblity(false)
            console.log("Plate is empty")
            parent.enabled=true
            addModel.markButtonAsActive()
            viewButton.markButtonAsActive()
            closeModelButtons()
            arrangeButton.markButtonAsNotActive()
            duplicateButton.markButtonAsNotActive()
            undoButton.markButtonAsNotActive()
            deactivateModelButtons()
        }

    }
    Connections
    {
        target:Gence.Actions.node_loading
        onTriggered: disableAll()
    }

    Connections
    {
        target:Gence.Actions.slicing
        onTriggered: disableAll()
    }
    Connections
    {
        target:Gence.Actions.split_deactivate_left_menu
        onTriggered:
        {
            Gence.Actions.activeAllTools.enabled=false
            parent.enabled=false

            moveButton.markButtonAsNotActive()
            scaleButton.markButtonAsNotActive()
            rotateButton.markButtonAsNotActive()
            duplicateButton.markButtonAsNotActive()

            addModel.markButtonAsNotActive()
            modelsListButton.markButtonAsNotActive()
            arrangeButton.markButtonAsNotActive()
            duplicateButton.markButtonAsNotActive()
            viewButton.markButtonAsNotActive()
            Gence.Actions.activeAllTools.enabled=false
        }
    }
    Connections
    {
        target:Gence.Actions.split_succ_activate_left_menu
        onTriggered:
        {
            Gence.Actions.node_loaded.trigger()
            moveButton.markButtonAsActive()
            duplicateButton.markButtonAsActive()
        }
    }

    Connections
    {
        target:Gence.Actions.split_fail_activate_left_menu
        onTriggered:
        {
            Gence.Actions.activeAllTools.enabled=true
            parent.enabled=true

            modelsListButton.markButtonAsActive()
            viewButton.markButtonAsActive()
            arrangeButton.markButtonAsActive()

            moveButton.markButtonAsActive()
            scaleButton.markButtonAsActive()
            rotateButton.markButtonAsActive()

            duplicateButton.markButtonAsActive()
        }
    }


    Connections
    {
        target:Gence.Actions.gcode_loading
        onTriggered: disableAll()
    }
    Connections
    {
        target:Gence.Actions.sliced
        onTriggered: showViewInLayerView()
    }
    Connections
    {
        target:Gence.Actions.gcode_loaded
        onTriggered: showViewInLayerView()
    }

    Connections
    {
        target: Gence.Actions.node_loaded
        onTriggered:
        {
            Gence.Actions.activeAllTools.enabled=true
            parent.enabled=true
            setModelsListVisiblity(true)
            // addModel.markButtonAsActive()
            modelsListButton.markButtonAsActive()
            viewButton.markButtonAsActive()
            arrangeButton.markButtonAsActive()
        }
    }
    Connections
    {
        target:UM.Selection
        onHasSelectionChanged:
        {
            if(UM.Selection.hasSingleSelection)
            {
                Gence.Actions.orderTranslateTool.trigger()
                // scaleButton.markButtonAsActive()
                // rotateButton.markButtonAsActive()
                // duplicateButton.markButtonAsActive()
                activateModelsOrMove()
            } else if(UM.Selection.hasSelection){
                modelsListButtonAction()
            }
        }
    }

    ColumnLayout {
        anchors.fill: parent
        //previous value = ... * 0.014, changed 19.01.2017
        spacing: parent.height * 0.008

        Logo {
            buttonAction.onClicked: Globals.modalWindow.openAboutWindow()
        }

        Spacer {}

        LeftMenuButton {
            id: addModel
            visible: true
            buttonText: "Add model"
            buttonIcon: "../../images/add_model.svg"
            buttonIconWhite: "../../images/add_model_white.svg"
            buttonAction.onClicked: openDialog.open()

            helpText: "Enables to locate and import a file in .stl format to the software. " +
                      "You can select several files to be imported simultaneously. You can also use " +
                      "the function of dragging .stl files onto the main software window."
        }

        ButtonWithMenu {
            id: modelsListButton
            visible: !addModel.visible
            buttonText: "Models"
            buttonIcon: "../../images/models_list.svg"
            buttonIconWhite: "../../images/models_list_white.svg"
            contentLoaderSource: "../../buttons/content/ModelsListButtonContent.qml"
            buttonAction.onClicked:{
                fakeDisableAnyTool()
                modelsListButtonAction()
            }

            menuHeight: width * 3.2
            menuWidth: width * 2.0
        }
        
        ButtonWithMenu {
            id: arrangeButton
            buttonText: "Arrange"
            buttonIcon: "../../images/arrange.svg"
            buttonIconWhite: "../../images/arrange_white.svg"
            contentLoaderSource: "../../buttons/content/ArrangeButtonContent.qml"
            buttonAction.onClicked: {
                fakeDisableAnyTool()
                arrangeButtonAction()
                modelsListOpened = false
            }
            menuHeight: width * 1.45
            menuWidth: width * 1.6
            helpText: "Place models on the heatbed automatically, choose the wall " +
            "that will become the basis of the print or enable advanced multimaterial options."
        }

        ButtonWithMenu{
            id: splitButton
            property string objectHelpText: ""
            property url objectHelpImageSource: ""
            buttonText: "Split"
            contentLoaderSource: "../../buttons/content/SplitButtonContent.qml"
            buttonAction.onClicked: Gence.Actions.orderMeshSplitterTool.trigger()
            menuHeight: item.advancedMode ? width * 3.65 : width * 2.15
            menuWidth: width * 1.6
            helpText: Help.splitButton
            helpImageSource: "../../images/help/SplitButton.png"
            menuHelpText: objectHelpText
            menuHelpImageSource: objectHelpImageSource
        }

        Spacer {}

        ButtonWithMenu {
            id: viewButton
            buttonText: "View"
            buttonIcon: "../../images/view.svg"
            buttonIconWhite: "../../images/view_white.svg"
            contentLoaderSource: "../../buttons/content/ViewButtonContent.qml"
            helpText: "Change camera settings"
            buttonAction.onClicked:
            {
                fakeDisableAnyTool()
                viewButtonAction()
                modelsListOpened = false
            }

            menuHeight:{
                if(appState == Gence.Notifications.SLICED || appState == Gence.Notifications.GCODE_LOADED){
                    return height * 4.2
                } else{
                    return height * 3.8
                }
            }
            menuWidth: {
                if(appState == Gence.Notifications.SLICED || appState == Gence.Notifications.GCODE_LOADED){
                    return width * 1.8
                } else{
                    return width * 1.7
                }
            }

            onIsOpenChanged:
            {
                if(isOpen)
                {
                    Gence.Actions.showMoveTool.enabled=true;
                }
            }
        }

        Spacer {}

        ButtonWithMenu {
            id: moveButton
            buttonText: "Move"
            buttonIcon: "../../images/move.svg"
            buttonIconWhite: "../../images/move_white.svg"
            contentLoaderSource: "../../buttons/content/MoveButtonContent.qml"
            helpText: "Move the selected model in the virtual workspace"
            buttonAction.onClicked: {
                modelsListOpened = false
                Gence.Actions.showMoveTool.enabled=true
                Gence.Actions.orderTranslateTool.trigger()
            }
                
            menuHeight: width * 2.0
            menuWidth: width * 1.6
        }

        ButtonWithMenu {
            id: scaleButton
            buttonText: "Scale"
            buttonIcon: "../../images/scale.svg"
            buttonIconWhite: "../../images/scale_white.svg"
            contentLoaderSource: "../../buttons/content/ScaleButtonContent.qml"
            helpText: "Change the model size"
            buttonAction.onClicked: {
                Gence.Actions.orderScaleTool.trigger()
                modelsListOpened = false
            }                
            menuHeight: width * 2.8
            menuWidth: width * 1.6
        }

        ButtonWithMenu {
            id: rotateButton
            buttonText: "Rotate"
            buttonIcon: "../../images/rotate.svg"
            buttonIconWhite: "../../images/rotate_white.svg"
            contentLoaderSource: "../../buttons/content/RotateButtonContent.qml"
            helpText: "Rotate the model"
            buttonAction.onClicked: {
                Gence.Actions.orderRotateTool.trigger()
                modelsListOpened = false
            }                
            menuHeight: width * 1.9
            menuWidth: width * 1.6
        }

        Spacer {}

        LeftMenuButton {
            id: duplicateButton
            buttonText: "Duplicate"
            buttonIcon: "../../images/duplicate.svg"
            buttonIconWhite: "../../images/duplicate_white.svg"
            helpText: "Clones the selected model and places it beside the original"
            buttonAction.onClicked: Gence.Actions.duplicateElement.trigger()
            cornerTriangleExists: false
        }

        Spacer {}

        ActionButton {
            enabled: ableToUndoNormal || ableToUndoAbnormal
            id:undoButton
            buttonText: "Undo"
            buttonAction.onClicked:
            {
                Gence.Actions.undo.trigger()
            }
            onEnabledChanged:
            {
                if(enabled)
                {
                    markButtonAsActive()
                }
                else
                {
                    markButtonAsNotActive()
                }
            }

            helpText: {
                if(ableToUndoNormal){
                    return  "Undoes most recent operations related to the models. " +
                            "If no operations on the solids were carried out after the " +
                            "model was loaded, pressing the button will remove the .stl " +
                            "model loaded most recently. In the .gcode preview mode, the " +
                            "button enables to return to the main software interface."
                } else if (ableToUndoAbnormal){
                    return  "Return to the main interface. After returning, it will be " +
                            "possible to edit the model and settings but before you continue " +
                            "you will have to prepare the .gcode file again using the Prepare " +
                            "to print button."
                } else {
                    return  ""
                }
            }
        }
    }


    UM.I18nCatalog {id: catalog; name:"cura"}
    FileDialog
    {
        id: openDialog;
        title: catalog.i18nc("@title:window","Open file")
        modality: UM.Application.platform == "linux" ? Qt.NonModal : Qt.WindowModal;
        selectMultiple: true
        nameFilters: UM.MeshFileHandler.supportedReadFileTypes;
        folder: CuraApplication.getDefaultPath("dialog_load_path")

        onAccepted:
        {
            var ftemp = folder;
            folder = ftemp;

            Gence.Actions.numberOfSelectedFiles=fileUrls.length

            CuraApplication.setDefaultPath("dialog_load_path", folder);
            for(var i in fileUrls)
            {
                Printer.readLocalFile(fileUrls[i])
            }
        }
    }

    Connections
    {
        target: Gence.Actions.showRotateTool
        onTriggered: rotateButtonAction()
    }

    Connections
    {
        target: Gence.Actions.showMoveTool
        onTriggered: activateModelsOrMove()
    }

    Connections
    {
        target:Gence.Actions.showScaleTool
        onTriggered: scaleButtonAction()
    }

    Connections
    {
        target: Gence.Actions.showSplitTool
        onTriggered: splitButtonAction()
    }

    Connections
    {
        target:Gence.Actions.hideAllTools
        onTriggered: closeModelButtons()
    }

    Connections
    {
        target:Gence.Actions.activeAllTools
        onTriggered: activateModelButtons()
    }

    Connections
    {
        target:Gence.Actions.disableAllTools
        onTriggered: deactivateModelButtons()
    }

    Connections
    {
        target: Gence.Actions.multipleModelsSelected
        onTriggered: deactivateToolButtons()
    }

    function fakeDisableAnyTool(){
        // COPYRIGHT: SOMEONE
        Gence.Actions.showMoveTool.enabled=false
        Gence.Actions.orderTranslateTool.trigger()
    }

    function arrangeButtonAction(){
        closeModelButtons()
        arrangeButton.openMenu()
    }

    function modelsListButtonAction(){
        closeModelButtons()
        modelsListOpened = true
        modelsListButton.openMenu()
    }

    function viewButtonAction(){
        closeModelButtons()
        viewButton.openMenu()
    }

    function moveButtonAction(){
        closeModelButtons()
        if (UM.Selection.hasSelection && !UM.Selection.hasSingleSelection){
            UM.Controller.activateSelectionTool(false)
        }
        moveButton.openMenu()
    }

    function activateModelsOrMove(){
        if(modelsListOpened === true){
            modelsListButtonAction()
        } else {
            moveButtonAction()
        }
    }

    function scaleButtonAction(){
        closeModelButtons()
        scaleButton.openMenu()
    }

    function splitButtonAction(){
        closeModelButtons()
        splitButton.openMenu()
    }

    function rotateButtonAction(){
        closeModelButtons()
        rotateButton.openMenu()
    }

    function deactivateToolButtons(){
        closeModelButtons()

        // moveButton.markButtonAsNotActive()
        scaleButton.markButtonAsNotActive()
        rotateButton.markButtonAsNotActive()
        splitButton.markButtonAsNotActive()
    }

    function deactivateModelButtons(){
        closeModelButtons()

        moveButton.markButtonAsNotActive()
        scaleButton.markButtonAsNotActive()
        rotateButton.markButtonAsNotActive()
        duplicateButton.markButtonAsNotActive()
        splitButton.markButtonAsNotActive()
    }

    function activateModelButtons(){
        closeModelButtons()

        moveButton.markButtonAsActive()
        scaleButton.markButtonAsActive()
        rotateButton.markButtonAsActive()
        duplicateButton.markButtonAsActive()
        splitButton.markButtonAsActive()
    }

    function closeModelButtons(){
        UM.Controller.activateSelectionTool(true)
        modelsListButton.closeMenu()
        viewButton.closeMenu()
        moveButton.closeMenu()
        scaleButton.closeMenu()
        rotateButton.closeMenu()
        arrangeButton.closeMenu()
        splitButton.closeMenu()
    }

    function showViewInLayerView(){
        parent.enabled=true;
        viewButton.markButtonAsActive()
        viewButtonAction()
    }

    function setModelsListVisiblity(state){
        addModel.visible = !state
        if(!state){
            modelsListButton.closeMenu()
        }
    }
}
