import QtQuick 2.5
import QtQuick.Window 2.2

import UM 1.3 as UM
import Cura 1.0 as Cura
import Gence 1.0 as Gence
import "layout"
import "singletons"

import "."

UM.MainWindow {
    id: base
    viewportRect: Qt.rect(0, 0, 0.85, 1.0)
    visible: true
    width: 1366
    height: 768

    title: qsTr("3DGence Slicer")

    UM.SettingPropertyProvider
    {
        id: supportEnabled

        containerStackId: Cura.MachineManager.activeMachineId
        key: "support_enable"
        watchedProperties: [ "value", "description" ]
        storeIndex: 0
    }

    Component.onCompleted:
    {
        Printer.setMinimumWindowSize(UM.Theme.getSize("window_minimum_size"))
        Cura.Actions.parent = mainForm
        //To be sure that ActiveTool is created before any tool is used
        UM.ActiveTool.valid
        Gence.Notifications.appState=Gence.Notifications.EMPTY_PLATE

        supportEnabled.setPropertyValue("value", true);
    }

    Component {
        id: startUpdate_comp

        Timer {
            interval: 1
            running: false
            repeat: false
            onTriggered: Printer.onStartCompleted ()
        }
    }

    MainForm {
        id: mainForm

        Loader {
            id: async_timer_loader
            asynchronous: true
            sourceComponent: startUpdate_comp
            onLoaded: {
                async_timer_loader.item.start();
                console.log("Now timer started!");
            }
        }

    }

    // turns on/off awareness area - it's a big mouse area that
    // is supposed to close active combo box as a reaction on
    // any outside user's action
    Connections {
        target            : combo_box_manager
        onComboBoxEnabled : {
            click_awareness_area.visible = true;
        }
    }

    // intercepts all mouse reactions that can close
    // currently shown component
    // ABOVE MainForm
    MouseArea {
        id                      : click_awareness_area;
        anchors.fill            : parent;
        visible                 : false;
        onClicked               : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        onPressed               : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        onReleased              : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        onDoubleClicked         : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        onPositionChanged       : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        onPressAndHold          : { mouse.accepted  = false; visible = false; combo_box_manager.forceHide (mouse.x, mouse.y); }
        propagateComposedEvents : true;
        z                       : mainForm.z + 1;
    }

    onBeforeRendering: function (){
        //if width has less place to put in 16:9 base window
        //See calculateViewDimension in MainWindow.py
        if(mainForm.width/16 <= mainForm.height/9) {
            Globals.baseWidth = mainForm.width
        } else {
            Globals.baseWidth = mainForm.height*(16/9)
        }
    }
}
