import QtQuick 2.4
import QtQuick.Controls 2.0
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1

import "../../singletons"
import "../../buttons"

Item {
    property alias checkBox: checkBox
    property alias updateStatus: updateStatus
    property int spacingSize: Globals.fontPixelSize * 0.5
    property string version
    property string additionalDBs
    
    Column {
        width: parent.width
        spacing: spacingSize

        Image {
            anchors.horizontalCenter: parent.horizontalCenter
            // mipmap: true

            fillMode: Image.PreserveAspectFit
            width: parent.width

            source: "../../images/logo.svg"
        }

        Text {
            font.pixelSize: Globals.fontPixelSize
            font.family: Fonts.boxed

            color: Colors.buttonOptionText
            text: "Gcode generator for 3DGence printers.<br>Version: " + version +
                  " <i>(Database" + (additionalDBs !== "" ? "s" : "") + ": Official" + additionalDBs + ")</i>" + 
                  "<br>Powered by CuraEngine developed by Ultimaker B.V."
        }

        CheckBox {
            id: checkBox
            width: box.width + label.contentWidth + spacing * 2
            height: Globals.controlHeight * 0.8

            indicator: Rectangle {
                id: box
                width: parent.height
                height: width
                border.color: Colors.customSettingsEditBorder

                Text {
                    id: line
                    font.pixelSize: Globals.fontPixelSize
                    // font.family: Fonts.boxed
                    color: Colors.customSettingsText
                    text: "✔"
                    anchors.centerIn: parent
                    visible: checkBox.checked
                }
            }

            contentItem: Label {
                id: label
                text: "Update profiles automatically"
                font.pixelSize: Globals.fontPixelSize
                font.family: Fonts.boxed
                color: Colors.buttonOptionText
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignRight
            }
        }

        Text {
            font.pixelSize: Globals.fontPixelSize
            font.family: Fonts.boxed

            color: Colors.buttonOptionText
            text: "<a href=\"licenses\">Licenses</a><br><a href=\"https://www.3dgence.com/support\">Support</a><br><a href=\"https://www.3dgence.com/support\">Manual</a>"

            onLinkActivated: {
                if (link === "licenses"){
                    licensesDialog.open()
                } else {
                    Qt.openUrlExternally(link)
                }
            }

            MouseArea {
                anchors.fill: parent
                acceptedButtons: Qt.NoButton
                cursorShape: parent.hoveredLink ? Qt.PointingHandCursor : Qt.ArrowCursor
            }
        }
    }

    Row {
        width: parent.width
        anchors.bottom: updateStatus.visible ? updateStatus.top : parent.bottom
        anchors.bottomMargin: updateStatus.visible ? spacingSize * 2 : 0
        spacing: Globals.fontPixelSize * 0.5
        MenuButton {
            width: parent.width/2 - parent.spacing/2
            menuButtonText: "Check for updates"
            buttonClickedAction.onTriggered: {
                updateStatus.visible = true
                Printer.checkForUpdate()
            }
        }
        MenuButton {
            width: parent.width/2 - parent.spacing/2
            menuButtonText: "Close"
            buttonClickedAction.onTriggered: close()
        }
    }

    Text {
        id: updateStatus
        visible: false
        font.pixelSize: Globals.fontPixelSize
        font.family: Fonts.boxed
        anchors.bottom: parent.bottom

        color: Colors.buttonOptionText
        text: Printer.updateStatusMsg
    }

    MessageDialog {
        id: licensesDialog
        title: "Licenses"
        icon: StandardIcon.Information
        text: "Please contact <a href=\"mailto:support@3dgence.com\">support@3dgence.com</a> for more information."
        standardButtons: StandardButton.Ok
    }
}