// Copyright (c) 2015 Ultimaker B.V.
// Uranium is released under the terms of the AGPLv3 or higher.

import QtQuick 2.2
import QtQuick.Controls 1.2
import QtQuick.Controls.Styles 1.4

import "../../singletons"

SettingItem
{
    id: base

    Component.onCompleted: {
        updateParentHeight()
    }

    function updateParentHeight(){
        parent.height = input.height
    }

    contents: Item
    {
        id: control
        anchors.fill: parent

        MouseArea
        {
            id: mouseArea
            anchors.fill: input
            //hoverEnabled: true
            cursorShape: Qt.IBeamCursor
        }

        TextArea
        {
            id: input

            flickableItem.boundsBehavior: Flickable.StopAtBounds 

            height: editVisible ? model.type === "polygons" || model.type === "polygon" ? width / 4 : width / 2 : -Globals.spacing

            onHeightChanged: {
                updateParentHeight()
            }

            style: TextAreaStyle {
                textColor: Colors.customListText
                selectionColor: Colors.customListSelectedBackground
                selectedTextColor: Colors.customListSelectedText
                backgroundColor: Colors.enabledBackground

                decrementControl: null
                incrementControl: null

                frame: Rectangle {
                    border.width: 1
                    border.color: Colors.customSettingsEditBorder
                }

                scrollBarBackground: Item {
                    implicitWidth: Globals.scrollBarWidth * 3
                }

                handle: Item {
                    implicitWidth: Globals.scrollBarWidth * 3
                    Rectangle {
                        width: Globals.scrollBarWidth
                        anchors {
                            top: parent.top
                            bottom: parent.bottom
                            right: parent.right
                            rightMargin: 2
                        }
                        color: Colors.customListScrollBarHandle
                    }
                }

                handleOverlap: -2
            }

            anchors
            {
                left: parent.left
                right: parent.right
                verticalCenter: parent.verticalCenter
            }

            Keys.onReleased:
            {
                propertyProvider.setPropertyValue("value", text)
            }

            onEditingFinished:
            {
                deselect()
                propertyProvider.setPropertyValue("value", text)
            }

            font.family: Fonts.boxed
            font.pixelSize: Globals.fontPixelSize

            selectByMouse: true;
            
            Binding
            {
                target: input
                property: "text"
                value:  {
                    // Stacklevels
                    // 0: user  -> unsaved change
                    // 1: quality changes  -> saved change
                    // 2: quality
                    // 3: material  -> user changed material in materialspage
                    // 4: variant
                    // 5: machine_changes
                    // 6: machine
                    if ((base.resolve != "None" && base.resolve) && (stackLevel != 0) && (stackLevel != 1)) {
                        // We have a resolve function. Indicates that the setting is not settable per extruder and that
                        // we have to choose between the resolved value (default) and the global value
                        // (if user has explicitly set this).
                        return base.resolve;
                    } else {
                        return propertyProvider.properties.value;
                    }
                }
                when: !input.activeFocus
            }
        }
    }
}
