import QtQuick 2.0
import QtQuick.Layouts 1.1

import "../singletons"
import "../tweaks"
import "../buttons"

Item {
    property alias unitString: unit.text
    property alias textField: textInput
    property alias valueText : textInput.text
    property alias plusButtonAction: plusButton.buttonMouseArea
    property alias minusButtonAction: minusButton.buttonMouseArea

    property bool standalone: false
    property bool instant: true
    property bool validValue: true
    property bool roundToStep: false
    property string curaParam: ""

    property real value: 0
    property int precision: 0
    property int temporaryValue:0
    property real singleStep: 1
    property real minValue: -2000000000
    property real maxValue:  2000000000
    property bool wasTextChanged: false
    property bool deactivate: false

    property bool inside : false
    property bool acceptDot : false

    property bool specialMaxValueInUse: false
    property bool specialMinValueInUse: false
    property bool specialMaxValue: false
    property bool specialMinValue: false

    property string specialMaxValueText
    property string specialMinValueText

    Layout.fillWidth: true
    Layout.preferredHeight: Math.floor(Globals.fontPixelSize * 2.09)
    Layout.alignment: Qt.AlignCenter

    RowLayout {
        anchors.fill: parent
        spacing: 1
        Keys.onLeftPressed: event.accepted=true
        Keys.onRightPressed: event.accepted=true
        Keys.onUpPressed: event.accepted=true
        Keys.onDownPressed: event.accepted= true
        SquareButton {
            id:minusButton
            buttonEnabled: !deactivate

            buttonText: "-"
            //            buttonMouseArea.onClicked: temporaryValue =value- singleStep
            shadowVisible: !standalone
        }

        Rectangle {

            //White background
            id:rect
            Layout.fillHeight: true
            Layout.fillWidth: true
            color: Colors.spinBoxEdit

            //Mouse area which drive focus from unit text to TextInput
            MouseArea
            {
                width:parent.width
                height: parent.height
                visible: !deactivate

                onClicked:
                {
                    textInput.selectAll()
                    textInput.forceActiveFocus()
                }
            }
            //TextInput holding value and Text holding unit of spinbox
            RowLayout
            {
                Layout.fillHeight: true
                Layout.fillWidth: true
                spacing: 0
                visible: !deactivate

                anchors.verticalCenter: parent.verticalCenter
                anchors.horizontalCenter: parent.horizontalCenter
                TextInput {
                    id: textInput
                    activeFocusOnPress: !deactivate
                    horizontalAlignment: TextInput.AlignRight
                    font.pixelSize: Globals.fontPixelSize
                    font.family: Fonts.boxed
                    color: validValue ? Colors.spinBoxEditText : Colors.spinBoxEditTextNotValid
                    selectionColor: Colors.customListSelectedBackground
                    selectedTextColor: Colors.customListSelectedText
                    maximumLength: 9-unit.text.length
//
//                    onFocusChanged:
//                    {
//                        if(focus)
//                            cursorPosition=0
//                    }


                    //exclude all alphabetical chars from textinput
                    onTextChanged:
                    {
                        //We dont want recurency here
                        if(inside == false)
                        {
                            inside = true

                            if(!specialMinValueInUse && !specialMaxValueInUse){
                                unit.visible = true

                                var justNumbers = text
                                var isNegative = false
                                
                                //Check if value should be negative
                                if(justNumbers.charAt(0) === '-' && minValue < 0){
                                    isNegative = true
                                }

                                //In case when we want to display double numbers
                                if(acceptDot)
                                {
                                    //We want to accept both coma and dot but use only dot
                                    justNumbers = justNumbers.replace(/,/, '.')

                                    justNumbers = justNumbers.replace(/[^0-9.]/g,'')
                                    
                                    var index = justNumbers.indexOf(".")

                                    //We are deleting every additional dot in string
                                    if(index !== -1 && index + 1 < justNumbers.length )
                                    {
                                        var indexAfterDot = index + 1
                                        var leftSide = justNumbers.substring(0,indexAfterDot)
                                        var rightSide = justNumbers.substring(indexAfterDot,justNumbers.length).replace(/[^0-9]/g,'')

                                        rightSide = rightSide.slice(0, precision)

                                        justNumbers = leftSide + rightSide
                                    }

                                } else {
                                    justNumbers = text.replace(/[^0-9]/g,'')
                                }

                                if(justNumbers.charAt(0) == '.'){
                                    justNumbers = '0' + justNumbers
                                }

                                if(isNegative){
                                    justNumbers = '-' + justNumbers
                                }

                                text = justNumbers

                            } else {
                                unit.visible = false

                                if(specialMinValueInUse){
                                    text = specialMinValueText
                                }

                                if(specialMaxValueInUse){
                                    text = specialMaxValueText
                                }
                            }
                            inside = false
                        }
                    }
                }
                Text
                {
                    id:unit
                    horizontalAlignment: TextInput.AlignLeft
                    font.pixelSize: Globals.fontPixelSize
                    font.family: Fonts.boxed
                    color: validValue ? Colors.spinBoxEditText : Colors.spinBoxEditTextNotValid
                    text: ""
                }
            }
        }

        SquareButton {
            id:plusButton
            buttonEnabled: !deactivate
            buttonText: "+"
            //buttonMouseArea.onClicked: value = value + singleStep
            shadowVisible: !standalone
        }
    }

    Rectangle {
        anchors.centerIn: parent
        width: parent.width + 2
        height: parent.height + 2
        visible: deactivate
        color: Colors.buttonDeactivator
    }

    LightBottomShadow {
        visible: standalone
    }
}
