import QtQuick 2.2
import QtQuick.Controls 2.0

import "../singletons"

Item {
    property alias hoverIndicatorOpacity: hoverIndicator.opacity
    property alias helpMouseArea: helpMouseArea
    property alias helpPopup: helpPopup
    property string helpText: ""
    property alias helpImageSource: helpImageItem.source
    property alias hoverIndicatorEnabled: hoverIndicator.visible

    MouseArea {
        id: helpMouseArea
        anchors.fill: parent
        hoverEnabled: true
        acceptedButtons: Qt.RightButton | Qt.LeftButton
        propagateComposedEvents: true
        preventStealing: true
    }

    Rectangle {
        id: hoverIndicator
        anchors.fill: parent
        visible: true
        color: Colors.buttonHovered
        opacity: 0.0
    }

    Popup {
        id: helpPopup

        property int basePopupWidth: Globals.baseWidth * 0.20
        property double paddingRatio: 0.04
        property int helpPopupPadding: basePopupWidth*paddingRatio
        property int helpPopupWidth: {
            if (hackTextItem.paintedWidth === 0){
                // display no frame
                return 0
            } else if (hackTextItem.paintedWidth < basePopupWidth - helpPopupPadding*2){
                return Math.ceil(hackTextItem.paintedWidth + helpPopupPadding*2)
            } else {
                return basePopupWidth
            }
        }
        width: helpPopupWidth
        padding: helpPopupPadding
        height: helpPopupPadding*(helpImageItem.source != "" ? 3: 2) + helpTextItem.contentHeight + helpImageItem.height
        closePolicy: Popup.NoAutoClose
        background: Rectangle {
            anchors.fill: parent
            color: Colors.contextHelpBackground
            border {
                width: 1
                color: Colors.contextHelpBorder
            }
        }

        contentItem: Column {
            spacing: parent.padding
            Text {
                id: helpTextItem
                width: parent.width
                color: Colors.contextHelpText
                text: helpText
                wrapMode: Text.WordWrap
                horizontalAlignment: Text.AlignJustify
                font {
                    family: Fonts.boxed
                    pixelSize: Globals.helpFontPixelSize
                }
            }

            Image {
                id: helpImageItem
                width: parent.width
                source: ""
                fillMode: Image.PreserveAspectFit
            }
        }
    }

    // Used to check rendered text length, if shorter than popup width, popup width is set to fit text closely.
    Text {
        id: hackTextItem
        visible: false
        text: helpText
        font {
            family: Fonts.boxed
            pixelSize: Globals.helpFontPixelSize
        }
    }
}
